module.exports = [
  '_debugger.js'
  '_http_agent.js'
  '_http_client.js'
  '_http_common.js'
  '_http_incoming.js'
  '_http_outgoing.js'
  '_http_server.js'
  '_linklist.js'
  '_stream_duplex.js'
  '_stream_passthrough.js'
  '_stream_readable.js'
  '_stream_transform.js'
  '_stream_writable.js'
  '_tls_legacy.js'
  '_tls_wrap.js'
  'assert.js'
  'buffer.js'
  'child_process.js'
  'cluster.js'
  'console.js'
  'constants.js'
  'crypto.js'
  'dgram.js'
  'dns.js'
  'domain.js'
  'events.js'
  'freelist.js'
  'fs.js'
  'http.js'
  'https.js'
  'module.js'
  'net.js'
  'os.js'
  'path.js'
  'punycode.js'
  'querystring.js'
  'readline.js'
  'repl.js'
  'smalloc.js'
  'stream.js'
  'string_decoder.js'
  'sys.js'
  'timers.js'
  'tls.js'
  'tty.js'
  'url.js'
  'util.js'
  'vm.js'
  'zlib.js'
  'node.js'
]