/**
 * Localized lower case.
 */
export declare function localeLowerCase(str: string, locale: string): string;
/**
 * Lower case as a function.
 */
export declare function lowerCase(str: string): string;
