declare const _exports: ({
    new (result?: import("postcss").Result<import("postcss").Document | import("postcss").Root> | undefined): important;
} | {
    new (result?: import("postcss").Result<import("postcss").Document | import("postcss").Root> | undefined): trailingSlashComma;
})[];
export = _exports;
import important = require("./important");
import trailingSlashComma = require("./trailingSlashComma");
//# sourceMappingURL=index.d.ts.map