module.exports={A:{A:{"1":"B","2":"K D E F A nC"},B:{"1":"0 9 M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I","129":"C L"},C:{"1":"0 9 oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","2":"oC MC","260":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB","804":"J QB K D E F A B C L M rC sC"},D:{"1":"0 9 uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","260":"pB qB rB sB tB","388":"UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB","1412":"1 2 3 4 5 6 7 8 G N O P RB SB TB","1956":"J QB K D E F A B C L M"},E:{"1":"VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","129":"A B C L M G xC TC GC HC yC zC 0C UC","1412":"K D E F vC wC","1956":"J QB tC SC uC"},F:{"1":"0 hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"F 5C 6C","260":"cB dB eB fB gB","388":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB","1796":"7C 8C","1828":"B C GC lC 9C HC"},G:{"1":"VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC","129":"GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC","1412":"E CD DD ED FD","1956":"SC AD mC BD"},H:{"1828":"XD"},I:{"1":"I","388":"cD dD","1956":"MC J YD ZD aD bD mC"},J:{"1412":"A","1924":"D"},K:{"1":"H","2":"A","1828":"B C GC lC HC"},L:{"1":"I"},M:{"1":"FC"},N:{"1":"B","2":"A"},O:{"1":"IC"},P:{"1":"1 2 3 4 5 6 7 8 gD hD iD TC jD kD lD mD nD JC KC LC oD","260":"eD fD","388":"J"},Q:{"1":"pD"},R:{"1":"qD"},S:{"1":"sD","260":"rD"}},B:4,C:"CSS3 Border images",D:true};
