var _sliceInstanceProperty = require("core-js-pure/features/instance/slice.js");
function _taggedTemplateLiteralLoose(e, t) {
  return t || (t = _sliceInstanceProperty(e).call(e, 0)), e.raw = t, e;
}
module.exports = _taggedTemplateLiteralLoose, module.exports.__esModule = true, module.exports["default"] = module.exports;