import _Object$defineProperty from "core-js-pure/features/object/define-property.js";
function _defineAccessor(e, r, n, t) {
  var c = {
    configurable: !0,
    enumerable: !0
  };
  return c[e] = t, _Object$defineProperty(r, n, c);
}
export { _defineAccessor as default };