module.exports={A:{A:{"1":"A B","2":"K D E F nC"},B:{"1":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I","2":"C L M G N O P"},C:{"1":"0 9 bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","2":"oC MC rC sC","132":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB"},D:{"1":"0 7 8 9 SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","2":"J","16":"2 3 4 5 6 QB K D E","132":"1 F A B C L M G N O P RB"},E:{"1":"C L M G GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","2":"J QB tC SC uC","132":"K D E F A B vC wC xC TC"},F:{"1":"0 1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"F B C 5C 6C 7C 8C GC lC 9C HC"},G:{"2":"CD DD","132":"E ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC","514":"SC AD mC BD"},H:{"2":"XD"},I:{"2":"YD ZD aD","260":"MC J bD mC","514":"I cD dD"},J:{"132":"A","260":"D"},K:{"2":"A B C GC lC HC","514":"H"},L:{"260":"I"},M:{"2":"FC"},N:{"514":"A","1028":"B"},O:{"2":"IC"},P:{"260":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"260":"pD"},R:{"260":"qD"},S:{"1":"rD sD"}},B:1,C:"accept attribute for file input",D:true};
