/**
List of binary file extensions.

@example
```
import binaryExtensions = require('binary-extensions');

console.log(binaryExtensions);
//=> ['3ds', '3g2', …]
```
*/
declare const binaryExtensions: readonly string[];

export = binaryExtensions;
