import Settings from '../../settings';
import { EntryTransformerFunction } from '../../types';
export default class EntryTransformer {
    private readonly _settings;
    constructor(_settings: Settings);
    getTransformer(): EntryTransformerFunction;
    private _transform;
}
