import type {MacroKeywordDefinition} from "ajv"
import type {GetDefinition} from "./_types"
import getRangeDef from "./_range"

const getDef: GetDefinition<MacroKeywordDefinition> = getRangeDef("exclusiveRange")

export default getDef
module.exports = getDef
