/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.themedImage {
  display: none;
}

[data-theme='light'] .themedImage--light {
  display: initial;
}

[data-theme='dark'] .themedImage--dark {
  display: initial;
}
