/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.tabList {
  margin-bottom: var(--ifm-leading);
}

.tabItem {
  margin-top: 0 !important;
}
