"use strict";

import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("renderRoutes");

import { renderRoutes } from "../esm/react-router-config.js";
export default renderRoutes;
