import _sliceInstanceProperty from "core-js-pure/features/instance/slice.js";
function _taggedTemplateLiteralLoose(e, t) {
  return t || (t = _sliceInstanceProperty(e).call(e, 0)), e.raw = t, e;
}
export { _taggedTemplateLiteralLoose as default };