'use strict';
require('../../modules/es.object.to-string');
require('../../modules/es.map');
require('../../modules/es.map.group-by');
var path = require('../../internals/path');

module.exports = path.Map.groupBy;
