/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

/** The name of the version that's actively worked on (e.g. `website/docs`) */
export const CURRENT_VERSION_NAME = 'current';
/** All doc versions are stored here by version names */
export const VERSIONED_DOCS_DIR = 'versioned_docs';
/** All doc versioned sidebars are stored here by version names */
export const VERSIONED_SIDEBARS_DIR = 'versioned_sidebars';
/** The version names. Should 1-1 map to the content of versioned docs dir. */
export const VERSIONS_JSON_FILE = 'versions.json';
