/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.content {
  font-size: 85%;
  text-align: center;
  padding: 5px 0;
}

.content a {
  color: inherit;
  text-decoration: underline;
}
