/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.tags {
  display: inline;
}

.tag {
  margin: 0 0.4rem 0.5rem 0;
  display: inline-block;
}
