/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.yearGroupHeading {
  margin: 1rem 0.75rem 0.5rem;
}
