module.exports={A:{A:{"2":"F A B nC","8":"K D E"},B:{"2":"C L M G N O P","8":"Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"0 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"1":"0 1 2 3 4 5 6 7 8 9 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","129":"oC MC rC sC"},D:{"1":"5","8":"1 2 3 4 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f","584":"g h i j k l m n o p q r","1025":"0 9 s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"1":"A B C L M G TC GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","260":"J QB K D E F tC SC uC vC wC xC"},F:{"2":"F","8":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC","584":"S T U V W X Y Z a b c d","1025":"0 e f g h i j k l m n o p q r s t u v w x y z","2052":"B C 5C 6C 7C 8C GC lC 9C HC"},G:{"1":"E BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC","8":"SC AD mC"},H:{"8":"XD"},I:{"8":"MC J YD ZD aD bD mC cD dD","1025":"I"},J:{"1":"A","8":"D"},K:{"8":"A B C GC lC HC","1025":"H"},L:{"1025":"I"},M:{"1":"FC"},N:{"2":"A B"},O:{"8":"IC"},P:{"1":"2 3 4 5 6 7 8","8":"1 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"8":"pD"},R:{"8":"qD"},S:{"1":"rD sD"}},B:2,C:"MathML",D:true};
