/**
Check if a project is using [Yarn](https://yarnpkg.com).

@param cwd - The current working directory. Default: `process.cwd()`.
@returns Whether the project uses Yarn.
*/
export default function (cwd?: string): boolean;
