import type {Plugin} from "ajv"
import getDef from "../definitions/patternRequired"

const patternRequired: Plugin<undefined> = (ajv) => ajv.addKeyword(getDef())

export default patternRequired
module.exports = patternRequired
