import React from 'react';

/* eslint-disable react/jsx-one-expression-per-line */
export default function SharedMultilevel() {
  return (
    <ul>
      <li>
        this is a <strong>shared multilevel</strong> component
      </li>
    </ul>
  );
}
/* eslint-enable react/jsx-one-expression-per-line */
