'use strict';
var parent = require('../../stable/iterator/for-each');
require('../../modules/esnext.iterator.constructor');
require('../../modules/esnext.iterator.for-each');

module.exports = parent;
