/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.listItem {
  background-color: #242526;
  padding: 10px;
  border-radius: 4px;
  margin-bottom: 20px;
}

.route {
  margin-bottom: 10px;
}

.routeName {
  color: #e06b6b;
}
