import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("MemoryRouter");

import { MemoryRouter } from "../esm/react-router-dom.js";
export default MemoryRouter;
