export type Check = import('./lib/index.js').Check
export type Test = import('./lib/index.js').Test
export type TestFunction = import('./lib/index.js').TestFunction
export {is, convert} from './lib/index.js'
