export default [
  require("/workspaces/airas/docs/node_modules/infima/dist/css/default/default.css"),
  require("/workspaces/airas/docs/node_modules/@docusaurus/theme-classic/lib/prism-include-languages"),
  require("/workspaces/airas/docs/node_modules/@docusaurus/theme-classic/lib/nprogress"),
  require("/workspaces/airas/docs/src/css/custom.css"),
];
