/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.tableOfContentsInline ul {
  list-style-type: disc;
  font-size: initial;
  padding-top: 0;
}
