export type Action = import('./lib/index.js').Action;
export type ActionTuple = import('./lib/index.js').ActionTuple;
export type Index = import('./lib/index.js').Index;
export type Visitor = import('./lib/index.js').Visitor;
export type Visitors = import('./lib/index.js').Visitors;
export { CONTINUE, EXIT, SKIP, visit } from "./lib/index.js";
