export declare class ValueHashError extends Error {
    readonly value: unknown;
    constructor(value: unknown);
}
export declare namespace ValueHash {
    /** Creates a FNV1A-64 non cryptographic hash of the given value */
    function Create(value: unknown): bigint;
}
