module.exports={A:{A:{"2":"K D E nC","2561":"A B","2692":"F"},B:{"1":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I","2561":"C L M G N O P"},C:{"1":"0 9 nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","16":"oC","1537":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB sC","1796":"MC rC"},D:{"1":"0 9 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","16":"J QB K D E F A B C L M","1025":"ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B","1537":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB"},E:{"1":"M G yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","16":"J QB K tC SC","1025":"D E F A B C vC wC xC TC GC","1537":"uC","4097":"L HC"},F:{"1":"0 qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z HC","16":"F B C 5C 6C 7C 8C GC lC","260":"9C","1025":"3 4 5 6 7 8 SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB","1537":"1 2 G N O P RB"},G:{"1":"OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC","16":"SC AD mC","1025":"E ED FD GD HD ID JD KD LD","1537":"BD CD DD","4097":"MD ND"},H:{"2":"XD"},I:{"16":"YD ZD","1025":"I dD","1537":"MC J aD bD mC cD"},J:{"1025":"A","1537":"D"},K:{"1":"A B C H GC lC HC"},L:{"1":"I"},M:{"1":"FC"},N:{"2561":"A B"},O:{"1":"IC"},P:{"1025":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"1":"pD"},R:{"1":"qD"},S:{"1":"sD","1537":"rD"}},B:1,C:"input event",D:true};
