/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

body:not(.navigation-with-keyboard) *:not(input):focus {
  outline: none;
}
