module.exports={A:{A:{"1":"B","4":"K D E F A nC"},B:{"1":"C L M G N","129":"0 9 O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"1":"1 2 3 4 5 6 7 8 K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB","4":"oC MC J QB rC sC","129":"0 9 pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC"},D:{"1":"jB kB lB mB nB oB pB qB rB sB","4":"J QB K","129":"0 1 2 3 4 5 6 7 8 9 D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"4":"J QB tC SC","129":"K D E F A B C L M G uC vC wC xC TC GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C"},F:{"1":"C WB XB YB ZB aB bB cB dB eB fB GC lC 9C HC","4":"F B 5C 6C 7C 8C","129":"0 1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z"},G:{"4":"SC AD mC","129":"E BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"4":"XD"},I:{"4":"YD ZD aD","129":"MC J I bD mC cD dD"},J:{"129":"D A"},K:{"1":"C GC lC HC","4":"A B","129":"H"},L:{"129":"I"},M:{"129":"FC"},N:{"1":"B","4":"A"},O:{"129":"IC"},P:{"129":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"129":"pD"},R:{"129":"qD"},S:{"1":"rD","129":"sD"}},B:1,C:"dataset & data-* attributes",D:true};
