import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("BrowserRouter");

import { BrowserRouter } from "../esm/react-router-dom.js";
export default BrowserRouter;
