import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("withRouter");

import { withRouter } from "../esm/react-router.js";
export default withRouter;
