export { fromJs } from "./lib/index.js";
export type Options = import('./lib/index.js').Options;
export type Version = import('./lib/index.js').Version;
export type Plugin = import('./lib/index.js').Plugin;
export type Value = import('./lib/index.js').Value;
