/**
List of Markdown file extensions.

@example
```
import markdownExtensions from 'markdown-extensions';

console.log(markdownExtensions);
//=> ['md', …]
```
*/
declare const markdownExtensions: [
	'md',
	'markdown',
	'mdown',
	'mkdn',
	'mkd',
	'mdwn',
	'mkdown',
	'ron',
];

export default markdownExtensions;
