'use strict';

const create = require('./create.cjs');
const parserSelector = require('../syntax/config/parser-selector.cjs');

const parseSelector = create.createParser(parserSelector);

module.exports = parseSelector;
