export {frontmatter} from './lib/syntax.js'
export {frontmatterHtml} from './lib/html.js'
export {toMatters} from './lib/to-matters.js'
export type Info = import('./lib/to-matters.js').Info
export type Matter = import('./lib/to-matters.js').Matter
export type Options = import('./lib/to-matters.js').Options
export type Preset = import('./lib/to-matters.js').Preset
