'use strict';
var parent = require('../../stable/json');
require('../../modules/es.object.create');
require('../../modules/es.object.freeze');
require('../../modules/es.object.keys');
require('../../modules/esnext.json.is-raw-json');
require('../../modules/esnext.json.parse');
require('../../modules/esnext.json.raw-json');

module.exports = parent;
