import type {Plugin} from "ajv"
import getDef from "../definitions/exclusiveRange"

const exclusiveRange: Plugin<undefined> = (ajv) => ajv.addKeyword(getDef())

export default exclusiveRange
module.exports = exclusiveRange
