module.exports={A:{A:{"2":"K D E F A B nC"},B:{"2":"C L M G N O P","2052":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"2":"oC MC J QB rC sC","1028":"0 9 aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","1060":"1 2 3 4 5 6 7 8 K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB"},D:{"2":"1 2 3 4 5 6 J QB K D E F A B C L M G N O P RB","226":"7 8 SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB","2052":"0 9 vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"2":"J QB K D tC SC uC vC","772":"L M G HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","804":"E F A B C xC TC GC","1316":"wC"},F:{"2":"1 2 3 4 5 6 7 8 F B C G N O P RB SB TB UB VB WB XB YB 5C 6C 7C 8C GC lC 9C HC","226":"ZB aB bB cB dB eB fB gB hB","2052":"0 iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z"},G:{"2":"SC AD mC BD CD DD","292":"E ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"1":"I","2":"MC J YD ZD aD bD mC cD dD"},J:{"2":"D A"},K:{"2":"A B C GC lC HC","2052":"H"},L:{"2052":"I"},M:{"1028":"FC"},N:{"2":"A B"},O:{"2052":"IC"},P:{"2":"J eD fD","2052":"1 2 3 4 5 6 7 8 gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"2052":"pD"},R:{"2052":"qD"},S:{"1028":"rD sD"}},B:4,C:"text-decoration styling",D:true};
