/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.title {
  font-size: 3rem;
}

/**
  Blog post title should be smaller on smaller devices
**/
@media (max-width: 576px) {
  .title {
    font-size: 2rem;
  }
}
