/**
 * Turns ReadableStream into async iterable when the `Symbol.asyncIterable` is not implemented on given stream.
 *
 * @param source A ReadableStream to create async iterator for
 */
export declare const getStreamIterator: (source: ReadableStream<Uint8Array> | AsyncIterable<Uint8Array>) => AsyncIterable<Uint8Array>;
