/**
 * CamelCase options.
 */
export interface CamelCaseOptions {
    reactCompat?: boolean;
}
/**
 * CamelCases a CSS property.
 */
export declare const camelCase: (property: string, options?: CamelCaseOptions) => string;
//# sourceMappingURL=utilities.d.ts.map