/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.wordWrapButtonIcon {
  width: 1.2rem;
  height: 1.2rem;
}

.wordWrapButtonEnabled .wordWrapButtonIcon {
  color: var(--ifm-color-primary);
}
