export type Callback = import('./lib/index.js').Callback;
export type Middleware = import('./lib/index.js').Middleware;
export type Pipeline = import('./lib/index.js').Pipeline;
export type Run = import('./lib/index.js').Run;
export type Use = import('./lib/index.js').Use;
export { trough, wrap } from "./lib/index.js";
//# sourceMappingURL=index.d.ts.map