import type {MacroKeywordDefinition} from "ajv"
import type {GetDefinition} from "./_types"
import getRequiredDef from "./_required"

const getDef: GetDefinition<MacroKeywordDefinition> = getRequiredDef("oneRequired")

export default getDef
module.exports = getDef
