
declare function defineDataProperty(
    obj: Record<PropertyKey, unknown>,
    property: keyof typeof obj,
    value: typeof obj[typeof property],
    nonEnumerable?: boolean | null,
    nonWritable?: boolean | null,
    nonConfigurable?: boolean | null,
    loose?: boolean
): void;

export = defineDataProperty;