/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

[data-theme='dark'] .threadsSvg {
  fill: var(--light);
}

[data-theme='light'] .threadsSvg {
  fill: var(--dark);
}
