/**
 * @typedef {import('./lib/index.js').Options} Options
 * @typedef {import('./lib/index.js').Version} Version
 * @typedef {import('./lib/index.js').Plugin} Plugin
 * @typedef {import('./lib/index.js').Value} Value
 */

export {fromJs} from './lib/index.js'
