/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

declare module 'postcss-sort-media-queries' {
  const plugin: import('postcss').PluginCreator<object>;
  export default plugin;
}
