module.exports={A:{A:{"2":"K D E F A B nC"},B:{"1":"0 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"1":"0 5 6 7 8 9 SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","33":"1 2 3 4 oC MC J QB K D E F A B C L M G N O P RB rC sC"},D:{"1":"0 9 bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","33":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB"},E:{"1":"F A B C L M G xC TC GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","33":"J QB K D E tC SC uC vC wC"},F:{"1":"0 5 6 7 8 C SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 9C HC","2":"F B 5C 6C 7C 8C GC lC","33":"1 2 3 4 G N O P RB"},G:{"2":"E SC AD mC BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"1":"I","2":"MC J YD ZD aD bD mC cD dD"},J:{"33":"D A"},K:{"1":"H","2":"A B C GC lC HC"},L:{"1":"I"},M:{"2":"FC"},N:{"2":"A B"},O:{"1":"IC"},P:{"2":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"1":"pD"},R:{"1":"qD"},S:{"2":"rD sD"}},B:2,C:"CSS3 Cursors: zoom-in & zoom-out",D:true};
