import type {KeywordDefinition} from "ajv"

export interface DefinitionOptions {
  defaultMeta?: string | boolean
}

export type GetDefinition<T extends KeywordDefinition> = (opts?: DefinitionOptions) => T
