import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("generatePath");

import { generatePath } from "../esm/react-router-dom.js";
export default generatePath;
