module.exports={A:{A:{"1":"A B","2":"F nC","8":"K D E"},B:{"1":"C L M G N O P Q H R S T","2":"0 9 U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"1":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S rC sC","2":"0 9 T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","4":"MC","8":"oC"},D:{"1":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T","2":"0 9 U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"1":"J QB K D E F A B C L M uC vC wC xC TC GC HC yC zC","2":"G 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","8":"tC SC"},F:{"1":"1 2 3 4 5 6 7 8 B C G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 8C GC lC 9C HC","2":"0 F 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 5C","8":"6C 7C"},G:{"1":"E SC AD mC BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD","2":"TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"1":"MC J YD ZD aD bD mC cD dD","2":"I"},J:{"1":"D A"},K:{"1":"B C GC lC HC","2":"A H"},L:{"2":"I"},M:{"2":"FC"},N:{"1":"A B"},O:{"1":"IC"},P:{"1":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"1":"pD"},R:{"2":"qD"},S:{"1":"rD","2":"sD"}},B:7,C:"Offline web applications",D:true};
