import React from 'react';

export default function Content() {
  return (
    <React.Fragment>
      <hr />
      Eu prosciutto fugiat, meatloaf beef ribs jerky dolore commodo est chicken t-bone meatball capicola magna ipsum. Ribeye shankle mollit venison.
    </React.Fragment>
  );
}
