module.exports={A:{A:{"1":"B","2":"K D E F A nC"},B:{"2":"0 9 C L M G N O P Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"1":"1 2 3 4 5 6 7 8 L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB","2":"0 9 oC MC J QB K D E F A B C pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC rC sC"},D:{"1":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB","2":"0 9 pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"1":"E F A B C xC TC GC","2":"J QB K D tC SC uC vC wC","129":"L M G HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C"},F:{"1":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB gB iB HC","2":"0 F B C eB fB hB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z 5C 6C 7C 8C GC lC 9C"},G:{"1":"E ED FD GD HD ID JD KD LD","2":"SC AD mC BD CD DD","257":"MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"1":"MC J bD mC cD dD","2":"I YD ZD aD"},J:{"2":"D A"},K:{"1":"HC","2":"A B C H GC lC"},L:{"2":"I"},M:{"2":"FC"},N:{"1":"B","2":"A"},O:{"2":"IC"},P:{"1":"J","2":"1 2 3 4 5 6 7 8 eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"2":"pD"},R:{"2":"qD"},S:{"1":"rD","2":"sD"}},B:7,C:"SPDY protocol",D:true};
