import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("matchPath");

import { matchPath } from "../esm/react-router-dom.js";
export default matchPath;
