module.exports={A:{A:{"2":"K D E nC","260":"F A B"},B:{"66":"IB JB KB LB MB NB OB PB I","132":"0 9 Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB","260":"C L M G N O P"},C:{"2":"oC MC J QB rC sC","260":"0 1 2 3 4 5 6 7 8 9 K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC"},D:{"16":"J QB K D E F A B C L M","66":"IB JB KB LB MB NB OB PB I QC FC RC","132":"0 1 2 3 4 5 6 7 8 9 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB"},E:{"16":"tC SC","132":"J QB K D E F A B C L M G uC vC wC xC TC GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C"},F:{"1":"C 9C HC","2":"F 5C 6C 7C 8C","16":"B GC lC","66":"0 w x y z","132":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v"},G:{"16":"SC AD","132":"E mC BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"16":"YD ZD","66":"I","132":"MC J aD bD mC cD dD"},J:{"132":"D A"},K:{"1":"C HC","2":"A","16":"B GC lC","132":"H"},L:{"66":"I"},M:{"260":"FC"},N:{"260":"A B"},O:{"132":"IC"},P:{"132":"1 2 3 4 5 6 7 8 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"132":"pD"},R:{"132":"qD"},S:{"260":"rD sD"}},B:7,C:"Mutation events",D:true};
