import { Linter } from "../index";

export interface StrictMode extends Linter.RulesRecord {
    /**
     * Rule to require or disallow strict mode directives.
     *
     * @since 0.1.0
     * @see https://eslint.org/docs/rules/strict
     */
    strict: Linter.RuleEntry<["safe" | "global" | "function" | "never"]>;
}
