export type Options = import('./lib/index.js').Options;
export type Matter = import('./lib/index.js').Matter;
export type Info = import('./lib/index.js').Info;
export { frontmatterFromMarkdown, frontmatterToMarkdown } from "./lib/index.js";
