export default {
  "17896441": [() => import(/* webpackChunkName: "17896441" */ "@theme/DocItem"), "@theme/DocItem", require.resolveWeak("@theme/DocItem")],
  "1dd53116": [() => import(/* webpackChunkName: "1dd53116" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/quickstart.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/quickstart.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/quickstart.md")],
  "1df93b7f": [() => import(/* webpackChunkName: "1df93b7f" */ "@site/src/pages/index.tsx"), "@site/src/pages/index.tsx", require.resolveWeak("@site/src/pages/index.tsx")],
  "1f391b9e": [() => import(/* webpackChunkName: "1f391b9e" */ "@theme/MDXPage"), "@theme/MDXPage", require.resolveWeak("@theme/MDXPage")],
  "2337773a": [() => import(/* webpackChunkName: "2337773a" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/MCP.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/MCP.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/development/MCP.md")],
  "2706947a": [() => import(/* webpackChunkName: "2706947a" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/executor.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/executor.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/executor.md")],
  "2938f627": [() => import(/* webpackChunkName: "2938f627" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/latex-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/latex-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/latex-subgraph.md")],
  "31e1897b": [() => import(/* webpackChunkName: "31e1897b" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/local-setup.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/local-setup.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/development/local-setup.md")],
  "36994c47": [() => import(/* webpackChunkName: "36994c47" */ "@generated/docusaurus-plugin-content-blog/default/__plugin.json"), "@generated/docusaurus-plugin-content-blog/default/__plugin.json", require.resolveWeak("@generated/docusaurus-plugin-content-blog/default/__plugin.json")],
  "393be207": [() => import(/* webpackChunkName: "393be207" */ "@site/src/pages/markdown-page.md"), "@site/src/pages/markdown-page.md", require.resolveWeak("@site/src/pages/markdown-page.md")],
  "42a77176": [() => import(/* webpackChunkName: "42a77176" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/congratulations.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/congratulations.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/congratulations.md")],
  "46fd0a4e": [() => import(/* webpackChunkName: "46fd0a4e" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-blog-post.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-blog-post.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-blog-post.md")],
  "5755394c": [() => import(/* webpackChunkName: "5755394c" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-document.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-document.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-document.md")],
  "5af6b628": [() => import(/* webpackChunkName: "5af6b628" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/intro.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/intro.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/intro.md")],
  "5e95c892": [() => import(/* webpackChunkName: "5e95c892" */ "@theme/DocsRoot"), "@theme/DocsRoot", require.resolveWeak("@theme/DocsRoot")],
  "5e9f5e1a": [() => import(/* webpackChunkName: "5e9f5e1a" */ "@generated/docusaurus.config"), "@generated/docusaurus.config", require.resolveWeak("@generated/docusaurus.config")],
  "5f477449": [() => import(/* webpackChunkName: "5f477449" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/markdown-features.mdx"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/markdown-features.mdx", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/markdown-features.mdx")],
  "60ea9faa": [() => import(/* webpackChunkName: "60ea9faa" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/html-uploader.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/html-uploader.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/html-uploader.md")],
  "62448f16": [() => import(/* webpackChunkName: "62448f16" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/roadmap.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/development/roadmap.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/development/roadmap.md")],
  "782a29ea": [() => import(/* webpackChunkName: "782a29ea" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-page.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-page.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/create-a-page.md")],
  "814f3328": [() => import(/* webpackChunkName: "814f3328" */ "~blog/default/blog-post-list-prop-default.json"), "~blog/default/blog-post-list-prop-default.json", require.resolveWeak("~blog/default/blog-post-list-prop-default.json")],
  "81904d0e": [() => import(/* webpackChunkName: "81904d0e" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/analytic-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/analytic-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/analytic-subgraph.md")],
  "82f84957": [() => import(/* webpackChunkName: "82f84957" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/readme-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/readme-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/readme-subgraph.md")],
  "8a2c5368": [() => import(/* webpackChunkName: "8a2c5368" */ "@generated/docusaurus-plugin-content-docs/default/p/airas-ja-docs-e48.json"), "@generated/docusaurus-plugin-content-docs/default/p/airas-ja-docs-e48.json", require.resolveWeak("@generated/docusaurus-plugin-content-docs/default/p/airas-ja-docs-e48.json")],
  "8daf86f5": [() => import(/* webpackChunkName: "8daf86f5" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/review-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/review-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/review-subgraph.md")],
  "8e3977a8": [() => import(/* webpackChunkName: "8e3977a8" */ "@site/docs/component/retrieve-paper-subgraph.md"), "@site/docs/component/retrieve-paper-subgraph.md", require.resolveWeak("@site/docs/component/retrieve-paper-subgraph.md")],
  "91c5a1a4": [() => import(/* webpackChunkName: "91c5a1a4" */ "@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-64b.json"), "@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-64b.json", require.resolveWeak("@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-64b.json")],
  "926a4862": [() => import(/* webpackChunkName: "926a4862" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/experimental-plan-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/experimental-plan-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/experimental-plan-subgraph.md")],
  "9e4087bc": [() => import(/* webpackChunkName: "9e4087bc" */ "@theme/BlogArchivePage"), "@theme/BlogArchivePage", require.resolveWeak("@theme/BlogArchivePage")],
  "9faed338": [() => import(/* webpackChunkName: "9faed338" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/research-preparation-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/research-preparation-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/research-preparation-subgraph.md")],
  "a5982edb": [() => import(/* webpackChunkName: "a5982edb" */ "@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-archive-612.json"), "@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-archive-612.json", require.resolveWeak("@generated/docusaurus-plugin-content-blog/default/p/airas-ja-blog-archive-612.json")],
  "a5c300e3": [() => import(/* webpackChunkName: "a5c300e3" */ "@site/blog/sample.md?truncated=true"), "@site/blog/sample.md?truncated=true", require.resolveWeak("@site/blog/sample.md?truncated=true")],
  "a6aa9e1f": [() => import(/* webpackChunkName: "a6aa9e1f" */ "@theme/BlogListPage"), "@theme/BlogListPage", require.resolveWeak("@theme/BlogListPage")],
  "a7456010": [() => import(/* webpackChunkName: "a7456010" */ "@generated/docusaurus-plugin-content-pages/default/__plugin.json"), "@generated/docusaurus-plugin-content-pages/default/__plugin.json", require.resolveWeak("@generated/docusaurus-plugin-content-pages/default/__plugin.json")],
  "a7bd4aaa": [() => import(/* webpackChunkName: "a7bd4aaa" */ "@theme/DocVersionRoot"), "@theme/DocVersionRoot", require.resolveWeak("@theme/DocVersionRoot")],
  "a7be5cf0": [() => import(/* webpackChunkName: "a7be5cf0" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/introduction.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/introduction.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/introduction.md")],
  "a94703ab": [() => import(/* webpackChunkName: "a94703ab" */ "@theme/DocRoot"), "@theme/DocRoot", require.resolveWeak("@theme/DocRoot")],
  "aba21aa0": [() => import(/* webpackChunkName: "aba21aa0" */ "@generated/docusaurus-plugin-content-docs/default/__plugin.json"), "@generated/docusaurus-plugin-content-docs/default/__plugin.json", require.resolveWeak("@generated/docusaurus-plugin-content-docs/default/__plugin.json")],
  "acecf23e": [() => import(/* webpackChunkName: "acecf23e" */ "~blog/default/blogMetadata-default.json"), "~blog/default/blogMetadata-default.json", require.resolveWeak("~blog/default/blogMetadata-default.json")],
  "bd278d7a": [() => import(/* webpackChunkName: "bd278d7a" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/writer-subgraph.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/writer-subgraph.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/writer-subgraph.md")],
  "ccc49370": [() => import(/* webpackChunkName: "ccc49370" */ "@theme/BlogPostPage"), "@theme/BlogPostPage", require.resolveWeak("@theme/BlogPostPage")],
  "d25c4c03": [() => import(/* webpackChunkName: "d25c4c03" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/generator.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/component/generator.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/component/generator.md")],
  "d61fbe0a": [() => import(/* webpackChunkName: "d61fbe0a" */ "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/deploy-your-site.md"), "@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/deploy-your-site.md", require.resolveWeak("@site/i18n/ja/docusaurus-plugin-content-docs/current/tutorial-basics/deploy-your-site.md")],
  "dff1c289": [() => import(/* webpackChunkName: "dff1c289" */ "@site/docs/tutorial-extras/manage-docs-versions.md"), "@site/docs/tutorial-extras/manage-docs-versions.md", require.resolveWeak("@site/docs/tutorial-extras/manage-docs-versions.md")],
  "e44a2883": [() => import(/* webpackChunkName: "e44a2883" */ "@site/docs/tutorial-extras/translate-your-site.md"), "@site/docs/tutorial-extras/translate-your-site.md", require.resolveWeak("@site/docs/tutorial-extras/translate-your-site.md")],
  "f1ed3f7d": [() => import(/* webpackChunkName: "f1ed3f7d" */ "@site/blog/sample.md"), "@site/blog/sample.md", require.resolveWeak("@site/blog/sample.md")],};
