'use strict';
require('../../modules/es.set');
require('../../modules/es.set.is-superset-of.v2');
var entryUnbind = require('../../internals/entry-unbind');

module.exports = entryUnbind('Set', 'isSupersetOf');
