/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.navbarHideable {
  transition: transform var(--ifm-transition-fast) ease;
}

.navbarHidden {
  transform: translate3d(0, calc(-100% - 2px), 0);
}
