import type {Plugin} from "ajv"
import getDef from "../definitions/oneRequired"

const oneRequired: Plugin<undefined> = (ajv) => ajv.addKeyword(getDef())

export default oneRequired
module.exports = oneRequired
