/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.docRoot {
  display: flex;
  width: 100%;
}

.docsWrapper {
  display: flex;
  flex: 1 0 auto;
}
