import warnAboutDeprecatedESMImport from "./warnAboutDeprecatedESMImport.js";
warnAboutDeprecatedESMImport("StaticRouter");

import { StaticRouter } from "../esm/react-router-dom.js";
export default StaticRouter;
