module.exports={A:{A:{"2":"K D E F A B nC"},B:{"1":"Q H R S T U V W","2":"C L M G N O P","1025":"0 9 X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I"},C:{"2":"1 2 3 4 5 6 7 8 oC MC J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B rC sC","260":"0 9 AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC"},D:{"1":"AC BC CC DC EC Q H R S T U V W","2":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC","132":"xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B","1025":"0 9 X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC"},E:{"2":"J QB K D E F A B tC SC uC vC wC xC TC","772":"C L M G GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C"},F:{"1":"yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC","2":"1 2 3 4 5 6 7 8 F B C G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB 5C 6C 7C 8C GC lC 9C HC","132":"lB mB nB oB pB qB rB sB tB uB vB wB xB","1025":"0 BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z"},G:{"2":"E SC AD mC BD CD DD ED FD GD HD ID JD","772":"KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"2":"XD"},I:{"1":"I","2":"MC J YD ZD aD bD mC cD dD"},J:{"2":"D A"},K:{"2":"A B C GC lC HC","1025":"H"},L:{"1025":"I"},M:{"260":"FC"},N:{"2":"A B"},O:{"1":"IC"},P:{"1":"1 2 3 4 5 6 7 8 jD kD lD mD nD JC KC LC oD","2":"J eD fD gD","132":"hD iD TC"},Q:{"132":"pD"},R:{"1025":"qD"},S:{"2":"rD","260":"sD"}},B:7,C:"Feature Policy",D:true};
