module.exports={A:{A:{"2":"K D E F A B nC"},B:{"1":"0 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I","2":"C L M G N O P","1028":"W X","1540":"Q H R S T U V"},C:{"1":"0 9 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","2":"oC","164":"1 2 3 4 5 6 7 8 MC J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB rC sC","1540":"fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B"},D:{"1":"0 9 Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","292":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B","1028":"W X","1540":"5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V"},E:{"1":"G 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C","292":"J QB K D E F A B C tC SC uC vC wC xC TC GC","1540":"L M HC yC","3076":"zC"},F:{"1":"0 CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"F B C 5C 6C 7C 8C GC lC 9C HC","292":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB","1028":"AC BC","1540":"uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B"},G:{"1":"TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC","292":"E SC AD mC BD CD DD ED FD GD HD ID JD KD LD","1540":"MD ND OD PD QD RD","3076":"SD"},H:{"2":"XD"},I:{"1":"I","292":"MC J YD ZD aD bD mC cD dD"},J:{"292":"D A"},K:{"1":"H","2":"A B C GC lC HC"},L:{"1":"I"},M:{"1":"FC"},N:{"2":"A B"},O:{"1":"IC"},P:{"1":"1 2 3 4 5 6 7 8 nD JC KC LC oD","292":"J eD fD gD hD iD","1540":"TC jD kD lD mD"},Q:{"1540":"pD"},R:{"1":"qD"},S:{"1":"sD","1540":"rD"}},B:5,C:"CSS Logical Properties",D:true};
