export interface Options {
    splitRegexp?: RegExp | RegExp[];
    stripRegexp?: RegExp | RegExp[];
    delimiter?: string;
    transform?: (part: string, index: number, parts: string[]) => string;
}
/**
 * Normalize the string into something other libraries can manipulate easier.
 */
export declare function noCase(input: string, options?: Options): string;
