import type {Plugin} from "ajv"
import getDef from "../definitions/allRequired"

const allRequired: Plugin<undefined> = (ajv) => ajv.addKeyword(getDef())

export default allRequired
module.exports = allRequired
