/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.errorBoundaryError {
  white-space: pre-wrap;
  color: red;
}

.errorBoundaryFallback {
  color: red;
  padding: 0.55rem;
}
