'use strict';
require('../../../modules/es.string.fixed');
var getBuiltInPrototypeMethod = require('../../../internals/get-built-in-prototype-method');

module.exports = getBuiltInPrototypeMethod('String', 'fixed');
