module.exports={A:{A:{"388":"A B","900":"K D E F nC"},B:{"388":"C L M G N O P","641":"0 9 r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I","900":"Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q"},C:{"772":"0 9 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC pC qC","900":"1 2 3 4 5 6 7 8 oC MC J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B rC sC"},D:{"641":"0 9 r s t u v w x y z AB BB CB DB EB FB GB HB IB JB KB LB MB NB OB PB I QC FC RC","900":"1 2 3 4 5 6 7 8 J QB K D E F A B C L M G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB NC xB OC yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R S T U V W X Y Z a b c d e f g h i j k l m n o p q"},E:{"772":"A","900":"J QB K D E F B C L M G tC SC uC vC wC xC TC GC HC yC zC 0C UC VC IC 1C JC WC XC YC ZC aC 2C KC bC cC dC eC fC 3C LC gC hC iC jC kC 4C"},F:{"16":"F 5C","129":"B C 6C 7C 8C GC lC 9C HC","641":"0 d e f g h i j k l m n o p q r s t u v w x y z","900":"1 2 3 4 5 6 7 8 G N O P RB SB TB UB VB WB XB YB ZB aB bB cB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC Q H R PC S T U V W X Y Z a b c"},G:{"900":"E SC AD mC BD CD DD ED FD GD HD ID JD KD LD MD ND OD PD QD RD SD TD UC VC IC UD JC WC XC YC ZC aC VD KC bC cC dC eC fC WD LC gC hC iC jC kC"},H:{"129":"XD"},I:{"641":"I","900":"MC J YD ZD aD bD mC cD dD"},J:{"900":"D A"},K:{"129":"A B C GC lC HC","641":"H"},L:{"900":"I"},M:{"772":"FC"},N:{"388":"A B"},O:{"900":"IC"},P:{"641":"2 3 4 5 6 7 8","900":"1 J eD fD gD hD iD TC jD kD lD mD nD JC KC LC oD"},Q:{"900":"pD"},R:{"900":"qD"},S:{"772":"sD","900":"rD"}},B:2,C:"CSS page-break properties",D:true};
