/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

.authorCol {
  max-width: inherit !important;
}

.imageOnlyAuthorRow {
  display: flex;
  flex-flow: row wrap;
}

.imageOnlyAuthorCol {
  margin-left: 0.3rem;
  margin-right: 0.3rem;
}
