'use strict';
var parent = require('../../stable/iterator/to-array');
require('../../modules/esnext.iterator.constructor');
require('../../modules/esnext.iterator.to-array');

module.exports = parent;
